// ====================================================================
// This file is part of FlexibleSUSY.
//
// FlexibleSUSY is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// FlexibleSUSY is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with FlexibleSUSY.  If not, see
// <http://www.gnu.org/licenses/>.
// ====================================================================

#include "effective_couplings.hpp"
#include "error.hpp"
#include "wrappers.hpp"

namespace flexiblesusy {

namespace effective_couplings {

std::complex<double> scaling_function(double tau)
{
   std::complex<double> result;

   if (tau > 1) {
      result = -0.25 * Sqr(Log((1.0 + Sqrt(1.0 - 1.0 / tau)) /
         (1.0 - Sqrt(1.0 - 1.0 / tau))) - std::complex<double>(0,1) * Pi);
   } else if (tau == 1.0) {
      result = 0.25 * Sqr(Pi);
   } else {
      result = Sqr(ArcSin(Sqrt(tau)));
   }

   return result;
}

std::complex<double> AS0(double tau)
{
   return -(tau - scaling_function(tau)) / Sqr(tau);
}

std::complex<double> AS12(double tau)
{
   return 2.0 * (tau + (tau - 1.0) * scaling_function(tau)) / Sqr(tau);
}

std::complex<double> AS1(double tau)
{
   return -(2.0 * Sqr(tau) + 3.0 * tau + 3.0 * (2.0 * tau - 1.0)
      * scaling_function(tau)) / Sqr(tau);
}

std::complex<double> AP12(double tau)
{
   return scaling_function(tau) / tau;
}

std::complex<double> scalar_diphoton_fermion_loop(
   double m_decay, double m_loop)
{
   const double tau = 0.25 * Sqr(m_decay) / Sqr(m_loop);
   const double tau_min = 75.0;

   const std::map<double, std::complex<double> > data
      = get_scalar_fermion_loop_data();

   std::complex<double> result;
   if (tau < tau_min) {
      const bool do_linear_interp = false;
      if (do_linear_interp) {
         result = linear_interpolation(tau, data);
      } else {
         result = quadratic_interpolation(tau, data);
      }
   } else {
      // analytic expression in the m_loop -> 0 limit
      const std::complex<double> limit =
         -(Sqr(Log(4.0 * tau)) - Sqr(Pi)) / 18.0 - 2.0 * Log(4.0 * tau) / 3.0
         + 2.0 * Log(tau) + 22.0 / Log(4.0 * tau) + std::complex<double>(0,1)
         * Pi * (Log(4.0 * tau) / 3.0 + 2.) / 3.0;

      const std::complex<double> min_value = (data.lower_bound(tau_min))->second;

      const std::complex<double> constant = min_value
         + (Sqr(Log(4.0 * tau_min)) - Sqr(Pi)) / 18.0 + 2.0 * Log(4.0 * tau_min)
         / 3.0 - 2.0 * Log(tau_min) - 22.0 / Log(4.0 * tau_min)
         - std::complex<double>(0,1) * Pi * (Log(4.0 * tau_min) / 3.0 + 2.)
         / 3.0;

      result = limit + constant;
   }

   return result;
}

std::complex<double> pseudoscalar_diphoton_fermion_loop(
   double m_decay, double m_loop)
{
   const double tau = 0.25 * Sqr(m_decay) / Sqr(m_loop);
   const double tau_min = 75.0;

   const std::map<double, std::complex<double> > data
      = get_pseudoscalar_fermion_loop_data();

   std::complex<double> result;
   if (tau < tau_min) {
      const bool do_linear_interp = false;
      if (do_linear_interp) {
         result = linear_interpolation(tau, data);
      } else {
         result = quadratic_interpolation(tau, data);
      }
   } else {
      // analytic expression in the m_loop -> 0 limit
      const std::complex<double> limit =
         -(Sqr(Log(4.0 * tau)) - Sqr(Pi)) / 18.0 - 2.0 * Log(4.0 * tau) / 3.0
         + 2.0 * Log(tau) + 21.0 / Log(4.0 * tau) + std::complex<double>(0,1)
         * Pi * (Log(4.0 * tau) / 3.0 + 2.) / 3.0;

      const std::complex<double> min_value = (data.lower_bound(tau_min))->second;

      const std::complex<double> constant = min_value
         + (Sqr(Log(4.0 * tau_min)) - Sqr(Pi)) / 18.0 + 2.0 * Log(4.0 * tau_min)
         / 3.0 - 2.0 * Log(tau_min) - 21.0 / Log(4.0 * tau_min)
         - std::complex<double>(0,1) * Pi * (Log(4.0 * tau_min) / 3.0 + 2.)
         / 3.0;

      result = limit + constant;
   }

   return result;
}

std::complex<double> linear_interpolation(
   double x, const std::map<double,std::complex<double> >& data)
{
   auto right = data.upper_bound(x);

   if (right == data.begin()) {
      ++right;
   } else if (right == data.end()) {
      --right;
   }

   auto left = right;
   --left;

   const double x_left = (x - right->first) / (left->first - right->first);
   const double x_right = (x - left->first) / (right->first - left->first);

   return left->second * x_left + right->second * x_right;
}

std::complex<double> quadratic_interpolation(
   double x, const std::map<double,std::complex<double> >& data)
{
   auto right = data.upper_bound(x);
   auto begin = data.begin();

   if (right == begin) {
      ++right;
      ++right;
   } else if (right == ++begin) {
      ++right;
   } else if (right == data.end()) {
      --right;
   }

   auto center = right;
   --center;
   auto left = center;
   --left;

   const double x_left = (x - center->first) * (x - right->first)
      / ((left->first - center->first) * (left->first - right->first));
   const double x_center = (x - left->first) * (x - right->first)
      / ((center->first - left->first) * (center->first - right->first));
   const double x_right = (x - left->first) * (x - center->first)
      / ((right->first - left->first) * (right->first - center->first));

   return left->second * x_left + center->second * x_center
      + right->second * x_right;
}

std::map<double,std::complex<double> > get_scalar_fermion_loop_data()
{
   static const std::map<double,std::complex<double> > data = {
      {0.000000, std::complex<double>(-1.000000, 0.000000)},
      {2.500000e-02, std::complex<double>(-9.274410e-01, 0.000000)},
      {5.000000e-02, std::complex<double>(-8.699240e-01, 0.000000)},
      {7.500000e-02, std::complex<double>(-8.171770e-01, 0.000000)},
      {1.000000e-01, std::complex<double>(-7.673120e-01, 0.000000)},
      {1.250000e-01, std::complex<double>(-7.192000e-01, 0.000000)},
      {1.500000e-01, std::complex<double>(-6.724460e-01, 0.000000)},
      {2.000000e-01, std::complex<double>(-5.813970e-01, 0.000000)},
      {2.250000e-01, std::complex<double>(-5.366180e-01, 0.000000)},
      {2.500000e-01, std::complex<double>(-4.920900e-01, 0.000000)},
      {2.750000e-01, std::complex<double>(-4.476720e-01, 0.000000)},
      {3.000000e-01, std::complex<double>(-4.032190e-01, 0.000000)},
      {3.250000e-01, std::complex<double>(-3.586210e-01, 0.000000)},
      {3.750000e-01, std::complex<double>(-2.685470e-01, 0.000000)},
      {4.000000e-01, std::complex<double>(-2.228670e-01, 0.000000)},
      {4.250000e-01, std::complex<double>(-1.766110e-01, 0.000000)},
      {4.500000e-01, std::complex<double>(-1.296750e-01, 0.000000)},
      {4.750000e-01, std::complex<double>(-8.194230e-02, 0.000000)},
      {5.000000e-01, std::complex<double>(-3.329450e-02, 0.000000)},
      {5.250000e-01, std::complex<double>(1.639910e-02, 0.000000)},
      {5.500000e-01, std::complex<double>(6.728150e-02, 0.000000)},
      {5.750000e-01, std::complex<double>(1.195110e-01, 0.000000)},
      {6.000000e-01, std::complex<double>(1.732640e-01, 0.000000)},
      {6.250000e-01, std::complex<double>(2.287460e-01, 0.000000)},
      {6.500000e-01, std::complex<double>(2.861880e-01, 0.000000)},
      {6.750000e-01, std::complex<double>(3.458640e-01, 0.000000)},
      {7.000000e-01, std::complex<double>(4.080960e-01, 0.000000)},
      {7.500000e-01, std::complex<double>(5.418690e-01, 0.000000)},
      {7.750000e-01, std::complex<double>(6.144780e-01, 0.000000)},
      {8.000000e-01, std::complex<double>(6.918550e-01, 0.000000)},
      {8.250000e-01, std::complex<double>(7.749940e-01, 0.000000)},
      {8.500000e-01, std::complex<double>(8.652370e-01, 0.000000)},
      {8.750000e-01, std::complex<double>(9.644870e-01, 0.000000)},
      {9.000000e-01, std::complex<double>(1.075600, 0.000000)},
      {9.250000e-01, std::complex<double>(1.203210, 0.000000)},
      {9.500000e-01, std::complex<double>(1.355880, 0.000000)},
      {9.750000e-01, std::complex<double>(1.553630, 0.000000)},
      {9.775000e-01, std::complex<double>(1.577670, 0.000000)},
      {9.800000e-01, std::complex<double>(1.602870, 0.000000)},
      {9.825000e-01, std::complex<double>(1.629430, 0.000000)},
      {9.850000e-01, std::complex<double>(1.657590, 0.000000)},
      {9.875000e-01, std::complex<double>(1.687670, 0.000000)},
      {9.900000e-01, std::complex<double>(1.720150, 0.000000)},
      {9.925000e-01, std::complex<double>(1.755740, 0.000000)},
      {9.950000e-01, std::complex<double>(1.795700, 0.000000)},
      {9.975000e-01, std::complex<double>(1.842770, 0.000000)},
      {9.997500e-01, std::complex<double>(1.899920, 0.000000)},
      {9.999750e-01, std::complex<double>(1.908560, 0.000000)},
      {9.999975e-01, std::complex<double>(1.908160, 0.000000)},
      {1.000002, std::complex<double>(1.910400, 1.397050e-02)},
      {1.000025, std::complex<double>(1.915280, 1.661240e-04)},
      {1.000250, std::complex<double>(1.920980, 4.940420e-03)},
      {1.002500, std::complex<double>(1.974600, 4.992560e-02)},
      {1.005000, std::complex<double>(2.015800, 9.685620e-02)},
      {1.007500, std::complex<double>(2.049050, 1.413840e-01)},
      {1.010000, std::complex<double>(2.075740, 1.877710e-01)},
      {1.012500, std::complex<double>(2.098510, 2.294580e-01)},
      {1.015000, std::complex<double>(2.117730, 2.678450e-01)},
      {1.017500, std::complex<double>(2.134460, 3.082090e-01)},
      {1.020000, std::complex<double>(2.148530, 3.453210e-01)},
      {1.022500, std::complex<double>(2.160570, 3.955390e-01)},
      {1.025000, std::complex<double>(2.167440, 3.674170e-01)},
      {1.050000, std::complex<double>(2.217290, 7.401210e-01)},
      {1.075000, std::complex<double>(2.202960, 9.972400e-01)},
      {1.100000, std::complex<double>(2.160810, 1.209370)},
      {1.125000, std::complex<double>(2.104920, 1.386380)},
      {1.150000, std::complex<double>(2.041990, 1.536790)},
      {1.175000, std::complex<double>(1.977190, 1.667430)},
      {1.200000, std::complex<double>(1.910320, 1.781250)},
      {1.225000, std::complex<double>(1.844700, 1.879300)},
      {1.250000, std::complex<double>(1.780290, 1.968390)},
      {1.275000, std::complex<double>(1.717540, 2.045420)},
      {1.300000, std::complex<double>(1.655320, 2.112210)},
      {1.325000, std::complex<double>(1.598500, 2.177810)},
      {1.350000, std::complex<double>(1.541930, 2.235760)},
      {1.375000, std::complex<double>(1.487760, 2.287090)},
      {1.400000, std::complex<double>(1.435160, 2.334260)},
      {1.425000, std::complex<double>(1.383500, 2.373900)},
      {1.450000, std::complex<double>(1.335620, 2.416060)},
      {1.475000, std::complex<double>(1.289760, 2.453690)},
      {1.500000, std::complex<double>(1.243910, 2.487070)},
      {1.525000, std::complex<double>(1.201800, 2.518090)},
      {1.550000, std::complex<double>(1.143560, 2.542650)},
      {1.575000, std::complex<double>(1.119830, 2.573500)},
      {1.600000, std::complex<double>(1.079290, 2.595490)},
      {1.625000, std::complex<double>(1.041180, 2.616630)},
      {1.650000, std::complex<double>(1.007980, 2.644280)},
      {1.675000, std::complex<double>(9.728280e-01, 2.665720)},
      {1.700000, std::complex<double>(9.395430e-01, 2.683790)},
      {1.725000, std::complex<double>(9.076790e-01, 2.703480)},
      {1.750000, std::complex<double>(8.751620e-01, 2.718770)},
      {1.775000, std::complex<double>(8.451780e-01, 2.734570)},
      {1.800000, std::complex<double>(8.157810e-01, 2.749670)},
      {1.825000, std::complex<double>(7.869390e-01, 2.764300)},
      {1.850000, std::complex<double>(7.589250e-01, 2.777410)},
      {1.875000, std::complex<double>(7.321510e-01, 2.790010)},
      {1.900000, std::complex<double>(7.087210e-01, 2.802260)},
      {1.925000, std::complex<double>(6.804190e-01, 2.813950)},
      {1.950000, std::complex<double>(6.561150e-01, 2.824470)},
      {1.975000, std::complex<double>(6.278130e-01, 2.830280)},
      {2.000000, std::complex<double>(6.079530e-01, 2.844660)},
      {2.025000, std::complex<double>(5.851860e-01, 2.856120)},
      {2.050000, std::complex<double>(5.640790e-01, 2.863510)},
      {2.075000, std::complex<double>(5.433580e-01, 2.871930)},
      {2.100000, std::complex<double>(5.199450e-01, 2.878980)},
      {2.125000, std::complex<double>(5.012280e-01, 2.888570)},
      {2.150000, std::complex<double>(4.822080e-01, 2.895210)},
      {2.175000, std::complex<double>(4.629140e-01, 2.902580)},
      {2.200000, std::complex<double>(4.438430e-01, 2.909290)},
      {2.225000, std::complex<double>(4.234060e-01, 2.913920)},
      {2.250000, std::complex<double>(4.055230e-01, 2.925310)},
      {2.275000, std::complex<double>(3.903050e-01, 2.928750)},
      {2.300000, std::complex<double>(3.728230e-01, 2.933830)},
      {2.325000, std::complex<double>(3.569420e-01, 2.940510)},
      {2.350000, std::complex<double>(3.410110e-01, 2.945240)},
      {2.375000, std::complex<double>(3.252790e-01, 2.949580)},
      {2.400000, std::complex<double>(3.091420e-01, 2.955910)},
      {2.425000, std::complex<double>(2.919870e-01, 2.959440)},
      {2.450000, std::complex<double>(2.786050e-01, 2.964930)},
      {2.475000, std::complex<double>(2.642730e-01, 2.970060)},
      {2.500000, std::complex<double>(2.459080e-01, 2.978780)},
      {2.625000, std::complex<double>(1.852400e-01, 2.991680)},
      {2.750000, std::complex<double>(1.253350e-01, 3.008150)},
      {2.875000, std::complex<double>(7.046560e-02, 3.021720)},
      {3.000000, std::complex<double>(2.001250e-02, 3.035770)},
      {3.125000, std::complex<double>(-2.403420e-02, 3.044270)},
      {3.250000, std::complex<double>(-6.512050e-02, 3.053970)},
      {3.375000, std::complex<double>(-1.093980e-01, 3.060300)},
      {3.500000, std::complex<double>(-1.394160e-01, 3.068600)},
      {3.625000, std::complex<double>(-1.726980e-01, 3.075360)},
      {3.750000, std::complex<double>(-2.031350e-01, 3.081210)},
      {3.875000, std::complex<double>(-2.363260e-01, 3.087060)},
      {4.000000, std::complex<double>(-2.583920e-01, 3.092340)},
      {4.125000, std::complex<double>(-2.832940e-01, 3.095760)},
      {4.250000, std::complex<double>(-3.085850e-01, 3.100940)},
      {4.375000, std::complex<double>(-3.286430e-01, 3.103730)},
      {4.500000, std::complex<double>(-3.493200e-01, 3.108120)},
      {4.625000, std::complex<double>(-3.688020e-01, 3.110960)},
      {4.750000, std::complex<double>(-3.872730e-01, 3.114360)},
      {4.875000, std::complex<double>(-4.077860e-01, 3.121410)},
      {5.000000, std::complex<double>(-4.211210e-01, 3.119970)},
      {5.250000, std::complex<double>(-4.517070e-01, 3.125250)},
      {5.500000, std::complex<double>(-4.804080e-01, 3.130170)},
      {5.750000, std::complex<double>(-5.042630e-01, 3.134640)},
      {6.000000, std::complex<double>(-5.269960e-01, 3.138710)},
      {6.250000, std::complex<double>(-5.486190e-01, 3.142740)},
      {6.500000, std::complex<double>(-5.670900e-01, 3.147910)},
      {6.750000, std::complex<double>(-5.842780e-01, 3.150820)},
      {7.000000, std::complex<double>(-6.017080e-01, 3.157090)},
      {7.250000, std::complex<double>(-6.183280e-01, 3.158450)},
      {7.500000, std::complex<double>(-6.294920e-01, 3.160890)},
      {7.750000, std::complex<double>(-6.423680e-01, 3.163460)},
      {8.000000, std::complex<double>(-6.561920e-01, 3.167320)},
      {8.250000, std::complex<double>(-6.580500e-01, 3.111450)},
      {8.500000, std::complex<double>(-6.760020e-01, 3.173130)},
      {8.750000, std::complex<double>(-6.859640e-01, 3.176160)},
      {9.000000, std::complex<double>(-6.948940e-01, 3.179130)},
      {9.250000, std::complex<double>(-7.031180e-01, 3.182040)},
      {9.500000, std::complex<double>(-7.116720e-01, 3.184860)},
      {9.750000, std::complex<double>(-7.193760e-01, 3.187730)},
      {1.000000e+01, std::complex<double>(-7.263360e-01, 3.190610)},
      {1.025000e+01, std::complex<double>(-7.331110e-01, 3.193210)},
      {1.050000e+01, std::complex<double>(-7.384460e-01, 3.201870)},
      {1.075000e+01, std::complex<double>(-7.456170e-01, 3.198790)},
      {1.100000e+01, std::complex<double>(-7.508340e-01, 3.201790)},
      {1.125000e+01, std::complex<double>(-7.565260e-01, 3.203980)},
      {1.150000e+01, std::complex<double>(-7.611350e-01, 3.206560)},
      {1.175000e+01, std::complex<double>(-7.662840e-01, 3.209140)},
      {1.200000e+01, std::complex<double>(-7.707960e-01, 3.212030)},
      {1.225000e+01, std::complex<double>(-7.752710e-01, 3.214360)},
      {1.250000e+01, std::complex<double>(-7.790270e-01, 3.217150)},
      {1.275000e+01, std::complex<double>(-7.845720e-01, 3.220080)},
      {1.300000e+01, std::complex<double>(-7.818920e-01, 3.222210)},
      {1.325000e+01, std::complex<double>(-7.906490e-01, 3.224360)},
      {1.350000e+01, std::complex<double>(-7.936570e-01, 3.227110)},
      {1.375000e+01, std::complex<double>(-7.971300e-01, 3.229460)},
      {1.400000e+01, std::complex<double>(-7.946630e-01, 3.244450)},
      {1.425000e+01, std::complex<double>(-8.026810e-01, 3.238130)},
      {1.450000e+01, std::complex<double>(-8.058680e-01, 3.238630)},
      {1.475000e+01, std::complex<double>(-8.085860e-01, 3.239030)},
      {1.500000e+01, std::complex<double>(-8.102950e-01, 3.241340)},
      {1.525000e+01, std::complex<double>(-8.133750e-01, 3.243740)},
      {1.550000e+01, std::complex<double>(-8.150670e-01, 3.248200)},
      {1.575000e+01, std::complex<double>(-8.176110e-01, 3.249920)},
      {1.600000e+01, std::complex<double>(-8.202630e-01, 3.254450)},
      {1.625000e+01, std::complex<double>(-8.463510e-01, 3.191430)},
      {1.650000e+01, std::complex<double>(-8.235680e-01, 3.256100)},
      {1.675000e+01, std::complex<double>(-8.250930e-01, 3.257230)},
      {1.700000e+01, std::complex<double>(-8.273730e-01, 3.259390)},
      {1.725000e+01, std::complex<double>(-8.286090e-01, 3.273550)},
      {1.750000e+01, std::complex<double>(-8.305330e-01, 3.263970)},
      {1.775000e+01, std::complex<double>(-8.328030e-01, 3.265800)},
      {1.800000e+01, std::complex<double>(-8.338430e-01, 3.268230)},
      {1.825000e+01, std::complex<double>(-8.347070e-01, 3.271350)},
      {1.850000e+01, std::complex<double>(-8.358370e-01, 3.274640)},
      {1.875000e+01, std::complex<double>(-8.374380e-01, 3.274590)},
      {1.900000e+01, std::complex<double>(-8.384120e-01, 3.277190)},
      {1.925000e+01, std::complex<double>(-8.402010e-01, 3.278980)},
      {1.950000e+01, std::complex<double>(-8.408070e-01, 3.280950)},
      {1.975000e+01, std::complex<double>(-8.418340e-01, 3.283500)},
      {2.000000e+01, std::complex<double>(-8.424940e-01, 3.286340)},
      {2.025000e+01, std::complex<double>(-8.442360e-01, 3.287140)},
      {2.050000e+01, std::complex<double>(-8.449020e-01, 3.289190)},
      {2.075000e+01, std::complex<double>(-8.464230e-01, 3.291930)},
      {2.100000e+01, std::complex<double>(-8.462180e-01, 3.293560)},
      {2.125000e+01, std::complex<double>(-8.480760e-01, 3.296380)},
      {2.150000e+01, std::complex<double>(-8.472430e-01, 3.298700)},
      {2.175000e+01, std::complex<double>(-8.489340e-01, 3.299710)},
      {2.200000e+01, std::complex<double>(-8.497100e-01, 3.301270)},
      {2.225000e+01, std::complex<double>(-8.506200e-01, 3.303100)},
      {2.250000e+01, std::complex<double>(-8.487620e-01, 3.314010)},
      {2.275000e+01, std::complex<double>(-8.500720e-01, 3.306940)},
      {2.300000e+01, std::complex<double>(-8.519600e-01, 3.309770)},
      {2.325000e+01, std::complex<double>(-8.522630e-01, 3.310530)},
      {2.350000e+01, std::complex<double>(-8.510320e-01, 3.315750)},
      {2.375000e+01, std::complex<double>(-8.539750e-01, 3.314820)},
      {2.400000e+01, std::complex<double>(-8.527500e-01, 3.315770)},
      {2.425000e+01, std::complex<double>(-8.547330e-01, 3.318300)},
      {2.450000e+01, std::complex<double>(-8.545140e-01, 3.319950)},
      {2.475000e+01, std::complex<double>(-8.555860e-01, 3.322180)},
      {2.500000e+01, std::complex<double>(-8.530350e-01, 3.323310)},
      {2.625000e+01, std::complex<double>(-8.575340e-01, 3.333010)},
      {2.750000e+01, std::complex<double>(-8.585380e-01, 3.341910)},
      {2.875000e+01, std::complex<double>(-8.576420e-01, 3.356210)},
      {3.000000e+01, std::complex<double>(-8.597370e-01, 3.356690)},
      {3.125000e+01, std::complex<double>(-8.576430e-01, 3.374410)},
      {3.250000e+01, std::complex<double>(-8.592460e-01, 3.377930)},
      {3.375000e+01, std::complex<double>(-8.597280e-01, 3.383100)},
      {3.500000e+01, std::complex<double>(-8.593010e-01, 3.390200)},
      {3.625000e+01, std::complex<double>(-8.584100e-01, 3.398350)},
      {3.750000e+01, std::complex<double>(-8.507510e-01, 3.413200)},
      {3.875000e+01, std::complex<double>(-8.567330e-01, 3.412240)},
      {4.000000e+01, std::complex<double>(-8.522690e-01, 3.427060)},
      {4.125000e+01, std::complex<double>(-8.544010e-01, 3.427280)},
      {4.250000e+01, std::complex<double>(-8.477110e-01, 3.450660)},
      {4.375000e+01, std::complex<double>(-8.525840e-01, 3.438880)},
      {4.500000e+01, std::complex<double>(-8.502110e-01, 3.447640)},
      {4.625000e+01, std::complex<double>(-8.494860e-01, 3.453470)},
      {4.750000e+01, std::complex<double>(-8.488420e-01, 3.457370)},
      {4.875000e+01, std::complex<double>(-8.462250e-01, 3.463940)},
      {5.000000e+01, std::complex<double>(-8.456140e-01, 3.469520)},
      {5.250000e+01, std::complex<double>(-8.430690e-01, 3.481720)},
      {5.500000e+01, std::complex<double>(-8.408260e-01, 3.492080)},
      {5.750000e+01, std::complex<double>(-8.371300e-01, 3.504700)},
      {6.000000e+01, std::complex<double>(-8.325950e-01, 3.517420)},
      {6.250000e+01, std::complex<double>(-8.314450e-01, 3.524800)},
      {6.500000e+01, std::complex<double>(-8.274530e-01, 3.537040)},
      {6.750000e+01, std::complex<double>(-8.261580e-01, 3.543130)},
      {7.000000e+01, std::complex<double>(-8.183210e-01, 3.548650)},
      {7.250000e+01, std::complex<double>(-8.197600e-01, 3.558200)},
      {7.500000e+01, std::complex<double>(-8.177000e-01, 3.569170)},
      {7.750000e+01, std::complex<double>(-8.089130e-01, 3.589240)},
      {8.000000e+01, std::complex<double>(-8.104730e-01, 3.586730)},
      {8.250000e+01, std::complex<double>(-8.087020e-01, 3.595360)},
      {8.500000e+01, std::complex<double>(-7.978130e-01, 3.608560)},
      {8.750000e+01, std::complex<double>(-8.044710e-01, 3.610160)},
      {9.000000e+01, std::complex<double>(-8.006390e-01, 3.617100)},
      {9.250000e+01, std::complex<double>(-7.970650e-01, 3.627710)},
      {9.500000e+01, std::complex<double>(-7.956520e-01, 3.631140)},
      {9.750000e+01, std::complex<double>(-7.847550e-01, 3.642800)},
      {1.000000e+02, std::complex<double>(-7.842690e-01, 3.653610)},
      {1.025000e+02, std::complex<double>(-7.582520e-01, 3.667740)},
      {1.050000e+02, std::complex<double>(-7.831340e-01, 3.661320)},
      {1.075000e+02, std::complex<double>(-7.669900e-01, 3.660770)},
      {1.100000e+02, std::complex<double>(-7.781230e-01, 3.674210)},
      {1.125000e+02, std::complex<double>(-7.610390e-01, 3.686090)},
      {1.150000e+02, std::complex<double>(-7.753390e-01, 3.684740)},
      {1.175000e+02, std::complex<double>(-8.045650e-01, 3.641630)},
      {1.200000e+02, std::complex<double>(-7.666080e-01, 3.702040)},
      {1.225000e+02, std::complex<double>(-8.494620e-01, 3.466970)},
      {1.250000e+02, std::complex<double>(-7.661640e-01, 3.710060)},
      {1.375000e+02, std::complex<double>(-7.547590e-01, 3.735350)},
      {1.500000e+02, std::complex<double>(-7.468060e-01, 3.761270)},
      {1.625000e+02, std::complex<double>(-7.134580e-01, 3.796710)},
      {1.750000e+02, std::complex<double>(-7.325910e-01, 3.810560)},
      {1.875000e+02, std::complex<double>(-7.204100e-01, 3.827430)},
      {2.000000e+02, std::complex<double>(-6.431750e-01, 3.921010)},
      {2.125000e+02, std::complex<double>(-6.989200e-01, 3.867360)},
      {2.250000e+02, std::complex<double>(-6.881680e-01, 3.889280)},
      {2.375000e+02, std::complex<double>(-6.741300e-01, 3.913110)},
      {2.500000e+02, std::complex<double>(-6.784930e-01, 3.918510)},
      {2.625000e+02, std::complex<double>(-7.149390e-01, 3.886200)},
      {2.750000e+02, std::complex<double>(-6.761000e-01, 3.940910)},
      {2.875000e+02, std::complex<double>(-6.682460e-01, 3.959060)},
      {3.000000e+02, std::complex<double>(-6.624520e-01, 3.967190)},
      {3.125000e+02, std::complex<double>(-6.574130e-01, 3.985200)},
      {3.250000e+02, std::complex<double>(-6.452390e-01, 4.010600)},
      {3.375000e+02, std::complex<double>(-6.349790e-01, 4.009280)},
      {3.500000e+02, std::complex<double>(-6.265670e-01, 4.006580)},
      {3.625000e+02, std::complex<double>(-6.381220e-01, 4.031630)},
      {3.750000e+02, std::complex<double>(-6.195300e-01, 4.046400)},
      {3.875000e+02, std::complex<double>(-6.423240e-01, 4.058040)},
      {4.000000e+02, std::complex<double>(-5.624150e-01, 4.050670)},
      {4.125000e+02, std::complex<double>(-6.127920e-01, 4.072060)},
      {4.250000e+02, std::complex<double>(-6.192850e-01, 4.082630)},
      {4.375000e+02, std::complex<double>(-6.139200e-01, 4.096560)},
      {4.500000e+02, std::complex<double>(-5.866120e-01, 4.089830)},
      {4.625000e+02, std::complex<double>(-5.710360e-01, 4.087280)},
      {4.750000e+02, std::complex<double>(-4.207690e-01, 4.288790)},
      {4.875000e+02, std::complex<double>(-6.170350e-01, 4.133430)},
      {5.000000e+02, std::complex<double>(-6.029440e-01, 4.137850)},
      {5.250000e+02, std::complex<double>(-5.822620e-01, 4.161870)},
      {5.500000e+02, std::complex<double>(-5.991270e-01, 4.170990)},
      {5.750000e+02, std::complex<double>(-5.189840e-01, 4.206430)},
      {6.000000e+02, std::complex<double>(-5.957670e-01, 4.193760)},
      {6.250000e+02, std::complex<double>(-5.628110e-01, 4.227600)},
      {6.500000e+02, std::complex<double>(-5.219400e-01, 4.171480)},
      {6.750000e+02, std::complex<double>(-5.646360e-01, 4.240940)},
      {7.000000e+02, std::complex<double>(-5.745030e-01, 4.236090)},
      {7.250000e+02, std::complex<double>(-5.744090e-01, 4.258600)},
      {7.500000e+02, std::complex<double>(-5.671400e-01, 4.258510)},
      {7.750000e+02, std::complex<double>(-5.652500e-01, 4.264550)},
      {8.000000e+02, std::complex<double>(-5.420170e-01, 4.295980)},
      {8.250000e+02, std::complex<double>(-5.422580e-01, 4.314380)},
      {8.500000e+02, std::complex<double>(-6.051220e-01, 4.246880)},
      {8.750000e+02, std::complex<double>(-5.504890e-01, 4.326040)},
      {9.000000e+02, std::complex<double>(-5.520800e-01, 4.325450)},
      {9.250000e+02, std::complex<double>(-4.959200e-01, 4.258360)},
      {9.500000e+02, std::complex<double>(-5.521010e-01, 4.358470)},
      {9.750000e+02, std::complex<double>(-5.294560e-01, 4.380260)},
      {1.000000e+03, std::complex<double>(-5.083960e-01, 4.371040)},
      {1.025000e+03, std::complex<double>(-5.456840e-01, 4.384650)},
      {1.050000e+03, std::complex<double>(-5.435860e-01, 4.393050)},
      {1.075000e+03, std::complex<double>(-5.416760e-01, 4.401180)},
      {1.100000e+03, std::complex<double>(-5.546870e-01, 4.371810)},
      {1.125000e+03, std::complex<double>(-5.668080e-01, 4.371030)},
      {1.150000e+03, std::complex<double>(-5.364750e-01, 4.424370)},
      {1.175000e+03, std::complex<double>(-4.698540e-01, 4.476270)},
      {1.200000e+03, std::complex<double>(-4.618550e-01, 4.354330)},
      {1.225000e+03, std::complex<double>(-4.703910e-01, 4.449320)},
      {1.250000e+03, std::complex<double>(-5.300020e-01, 4.453420)},
      {1.375000e+03, std::complex<double>(-4.930750e-01, 4.494780)},
      {1.500000e+03, std::complex<double>(-5.172970e-01, 4.516980)},
      {1.625000e+03, std::complex<double>(-5.510960e-01, 4.559270)},
      {1.750000e+03, std::complex<double>(-5.078310e-01, 4.571030)},
      {1.875000e+03, std::complex<double>(-4.901370e-01, 4.622460)},
      {2.000000e+03, std::complex<double>(-4.839760e-01, 4.656640)},
      {2.125000e+03, std::complex<double>(-6.801030e-01, 4.470020)},
      {2.250000e+03, std::complex<double>(-4.950810e-01, 4.659700)},
      {2.375000e+03, std::complex<double>(-4.927970e-01, 4.678860)},
      {2.500000e+03, std::complex<double>(-4.907780e-01, 4.697060)},
      {2.625000e+03, std::complex<double>(-4.890080e-01, 4.714400)},
      {2.750000e+03, std::complex<double>(-4.874400e-01, 4.730940)},
      {2.875000e+03, std::complex<double>(-4.860670e-01, 4.746770)},
      {3.000000e+03, std::complex<double>(-4.848620e-01, 4.761940)},
      {3.125000e+03, std::complex<double>(-4.838120e-01, 4.776500)},
      {3.250000e+03, std::complex<double>(-4.828940e-01, 4.790510)},
      {3.375000e+03, std::complex<double>(-4.821040e-01, 4.803990)},
      {3.500000e+03, std::complex<double>(-4.814220e-01, 4.817000)},
      {3.625000e+03, std::complex<double>(-4.808480e-01, 4.829560)},
      {3.750000e+03, std::complex<double>(-4.803670e-01, 4.841700)},
      {3.875000e+03, std::complex<double>(-4.799690e-01, 4.853450)},
      {4.000000e+03, std::complex<double>(-4.796470e-01, 4.864840)},
      {4.125000e+03, std::complex<double>(-4.793990e-01, 4.875880)},
      {4.250000e+03, std::complex<double>(-4.792170e-01, 4.886590)},
      {4.375000e+03, std::complex<double>(-4.790840e-01, 4.896990)},
      {4.500000e+03, std::complex<double>(-4.789680e-01, 4.907040)},
      {4.625000e+03, std::complex<double>(-4.790210e-01, 4.916980)},
      {4.750000e+03, std::complex<double>(-4.790510e-01, 4.926570)},
      {4.875000e+03, std::complex<double>(-4.791300e-01, 4.935920)},
      {5.000000e+03, std::complex<double>(-4.792570e-01, 4.945040)},
      {5.250000e+03, std::complex<double>(-4.796050e-01, 4.962610)},
      {5.500000e+03, std::complex<double>(-4.800930e-01, 4.979390)},
      {5.750000e+03, std::complex<double>(-4.807000e-01, 4.995430)},
      {6.000000e+03, std::complex<double>(-4.814030e-01, 5.010780)},
      {6.250000e+03, std::complex<double>(-4.821960e-01, 5.025530)},
      {6.500000e+03, std::complex<double>(-4.830640e-01, 5.039710)},
      {6.750000e+03, std::complex<double>(-4.840040e-01, 5.053350)},
      {7.000000e+03, std::complex<double>(-4.850120e-01, 5.066510)},
      {7.250000e+03, std::complex<double>(-4.860650e-01, 5.079220)},
      {7.500000e+03, std::complex<double>(-4.871620e-01, 5.091490)},
      {7.750000e+03, std::complex<double>(-4.883080e-01, 5.103370)},
      {8.000000e+03, std::complex<double>(-4.894960e-01, 5.114870)},
      {8.250000e+03, std::complex<double>(-4.907040e-01, 5.126030)},
      {8.500000e+03, std::complex<double>(-4.919510e-01, 5.136860)},
      {8.750000e+03, std::complex<double>(-4.932090e-01, 5.147370)},
      {9.000000e+03, std::complex<double>(-4.945050e-01, 5.157590)},
      {9.250000e+03, std::complex<double>(-4.957630e-01, 5.167590)},
      {9.500000e+03, std::complex<double>(-4.971370e-01, 5.177230)},
      {9.750000e+03, std::complex<double>(-4.984880e-01, 5.186670)},
      {1.000000e+04, std::complex<double>(-4.998480e-01, 5.195860)},
      {1.025000e+04, std::complex<double>(-5.012260e-01, 5.204840)},
      {1.050000e+04, std::complex<double>(-5.026090e-01, 5.213590)},
      {1.075000e+04, std::complex<double>(-5.039980e-01, 5.222160)},
      {1.100000e+04, std::complex<double>(-5.053960e-01, 5.230520)},
      {1.125000e+04, std::complex<double>(-5.068090e-01, 5.238680)},
      {1.150000e+04, std::complex<double>(-5.082170e-01, 5.246680)},
      {1.175000e+04, std::complex<double>(-5.096310e-01, 5.254510)},
      {1.200000e+04, std::complex<double>(-5.110560e-01, 5.262180)},
      {1.225000e+04, std::complex<double>(-5.124760e-01, 5.269690)},
      {1.250000e+04, std::complex<double>(-5.139030e-01, 5.277040)},
      {1.375000e+04, std::complex<double>(-5.210650e-01, 5.311770)},
      {1.500000e+04, std::complex<double>(-5.281870e-01, 5.343490)},
      {1.625000e+04, std::complex<double>(-5.352570e-01, 5.372690)},
      {1.750000e+04, std::complex<double>(-5.407550e-01, 5.400700)},
      {1.875000e+04, std::complex<double>(-5.491820e-01, 5.424950)},
      {2.000000e+04, std::complex<double>(-5.559810e-01, 5.448540)},
      {2.125000e+04, std::complex<double>(-5.608930e-01, 5.471800)},
      {2.250000e+04, std::complex<double>(-5.651330e-01, 5.489130)},
      {2.375000e+04, std::complex<double>(-5.731510e-01, 5.507680)},
      {2.500000e+04, std::complex<double>(-5.416510e-01, 5.554760)},
      {2.625000e+04, std::complex<double>(-5.851030e-01, 5.551470)},
      {2.750000e+04, std::complex<double>(-5.911890e-01, 5.568540)},
      {2.875000e+04, std::complex<double>(-5.972020e-01, 5.584870)},
      {3.000000e+04, std::complex<double>(-6.030830e-01, 5.600500)},
      {3.125000e+04, std::complex<double>(-6.088740e-01, 5.615470)},
      {3.250000e+04, std::complex<double>(-6.145350e-01, 5.629900)},
      {3.375000e+04, std::complex<double>(-6.201340e-01, 5.643760)},
      {3.500000e+04, std::complex<double>(-6.256230e-01, 5.657140)},
      {3.625000e+04, std::complex<double>(-6.310320e-01, 5.670020)},
      {3.750000e+04, std::complex<double>(-6.363710e-01, 5.682500)},
      {3.875000e+04, std::complex<double>(-6.416170e-01, 5.694550)},
      {4.000000e+04, std::complex<double>(-6.467650e-01, 5.706240)},
      {4.125000e+04, std::complex<double>(-6.518280e-01, 5.717570)},
      {4.250000e+04, std::complex<double>(-6.567960e-01, 5.728560)},
      {4.375000e+04, std::complex<double>(-6.617620e-01, 5.739240)},
      {4.500000e+04, std::complex<double>(-6.667960e-01, 5.749480)},
      {4.625000e+04, std::complex<double>(-6.716260e-01, 5.759560)},
      {4.750000e+04, std::complex<double>(-6.762020e-01, 5.769430)},
      {4.875000e+04, std::complex<double>(-6.809970e-01, 5.778920)},
      {5.000000e+04, std::complex<double>(-6.854800e-01, 5.788330)},
      {5.250000e+04, std::complex<double>(-6.945120e-01, 5.806250)},
      {5.500000e+04, std::complex<double>(-7.029630e-01, 5.823580)},
      {5.750000e+04, std::complex<double>(-7.114860e-01, 5.839920)},
      {6.000000e+04, std::complex<double>(-7.201270e-01, 5.855470)},
      {6.250000e+04, std::complex<double>(-7.284420e-01, 5.870400)},
      {6.500000e+04, std::complex<double>(-7.363280e-01, 5.884850)},
      {6.750000e+04, std::complex<double>(-7.437560e-01, 5.898960)},
      {7.000000e+04, std::complex<double>(-7.516830e-01, 5.912240)},
      {7.250000e+04, std::complex<double>(-7.588050e-01, 5.925290)},
      {7.500000e+04, std::complex<double>(-7.662530e-01, 5.937730)},
      {7.750000e+04, std::complex<double>(-7.732090e-01, 5.949940)},
      {8.000000e+04, std::complex<double>(-7.803130e-01, 5.961600)},
      {8.250000e+04, std::complex<double>(-7.876450e-01, 5.972720)},
      {8.500000e+04, std::complex<double>(-7.940580e-01, 5.983890)},
      {8.750000e+04, std::complex<double>(-8.010120e-01, 5.994420)},
      {9.000000e+04, std::complex<double>(-8.070480e-01, 6.005040)},
      {9.250000e+04, std::complex<double>(-8.136710e-01, 6.015010)},
      {9.500000e+04, std::complex<double>(-8.199980e-01, 6.024830)},
      {9.750000e+04, std::complex<double>(-8.258620e-01, 6.034570)},
      {1.000000e+05, std::complex<double>(-8.317770e-01, 6.044000)},
      {1.025000e+05, std::complex<double>(-8.382350e-01, 6.052860)},
      {1.050000e+05, std::complex<double>(-8.444230e-01, 6.061530)},
      {1.075000e+05, std::complex<double>(-8.497830e-01, 6.070490)},
      {1.100000e+05, std::complex<double>(-8.555830e-01, 6.078880)},
      {1.125000e+05, std::complex<double>(-8.611510e-01, 6.087280)},
      {1.150000e+05, std::complex<double>(-8.665250e-01, 6.095480)},
      {1.175000e+05, std::complex<double>(-8.724370e-01, 6.103110)},
      {1.200000e+05, std::complex<double>(-8.772580e-01, 6.111240)},
      {1.225000e+05, std::complex<double>(-8.832730e-01, 6.118490)},
      {1.250000e+05, std::complex<double>(-8.883480e-01, 6.125990)},
      {1.375000e+05, std::complex<double>(-9.132800e-01, 6.161310)},
      {1.500000e+05, std::complex<double>(-9.370350e-01, 6.193350)},
      {1.625000e+05, std::complex<double>(-9.593780e-01, 6.222940)},
      {1.750000e+05, std::complex<double>(-9.803260e-01, 6.250430)},
      {1.875000e+05, std::complex<double>(-1.000830, 6.275730)},
      {2.000000e+05, std::complex<double>(-1.019750, 6.299570)},
      {2.125000e+05, std::complex<double>(-1.038250, 6.321990)},
      {2.250000e+05, std::complex<double>(-1.055860, 6.342960)},
      {2.375000e+05, std::complex<double>(-1.072980, 6.362930)},
      {2.500000e+05, std::complex<double>(-1.070330, 6.345460)}
   };

   return data;
}

std::map<double,std::complex<double> > get_pseudoscalar_fermion_loop_data()
{
   static const std::map<double,std::complex<double> > data = {
      {0.000000, std::complex<double>(0.000000, 0.000000)},
      {2.500000e-02, std::complex<double>(1.076560e-01, 0.000000)},
      {5.000000e-02, std::complex<double>(1.949420e-01, 0.000000)},
      {7.500000e-02, std::complex<double>(2.758870e-01, 0.000000)},
      {1.000000e-01, std::complex<double>(3.534260e-01, 0.000000)},
      {1.250000e-01, std::complex<double>(4.289350e-01, 0.000000)},
      {1.500000e-01, std::complex<double>(5.032720e-01, 0.000000)},
      {2.000000e-01, std::complex<double>(6.505730e-01, 0.000000)},
      {2.250000e-01, std::complex<double>(7.243250e-01, 0.000000)},
      {2.500000e-01, std::complex<double>(7.985640e-01, 0.000000)},
      {2.750000e-01, std::complex<double>(8.735650e-01, 0.000000)},
      {3.000000e-01, std::complex<double>(9.495800e-01, 0.000000)},
      {3.250000e-01, std::complex<double>(1.026860, 0.000000)},
      {3.750000e-01, std::complex<double>(1.186230, 0.000000)},
      {4.000000e-01, std::complex<double>(1.268850, 0.000000)},
      {4.250000e-01, std::complex<double>(1.353800, 0.000000)},
      {4.500000e-01, std::complex<double>(1.441410, 0.000000)},
      {4.750000e-01, std::complex<double>(1.532020, 0.000000)},
      {5.000000e-01, std::complex<double>(1.626020, 0.000000)},
      {5.250000e-01, std::complex<double>(1.723870, 0.000000)},
      {5.500000e-01, std::complex<double>(1.826080, 0.000000)},
      {5.750000e-01, std::complex<double>(1.933240, 0.000000)},
      {6.000000e-01, std::complex<double>(2.046040, 0.000000)},
      {6.250000e-01, std::complex<double>(2.165330, 0.000000)},
      {6.500000e-01, std::complex<double>(2.292110, 0.000000)},
      {6.750000e-01, std::complex<double>(2.427600, 0.000000)},
      {7.000000e-01, std::complex<double>(2.573330, 0.000000)},
      {7.500000e-01, std::complex<double>(2.903840, 0.000000)},
      {7.750000e-01, std::complex<double>(3.094420, 0.000000)},
      {8.000000e-01, std::complex<double>(3.307510, 0.000000)},
      {8.250000e-01, std::complex<double>(3.549500, 0.000000)},
      {8.500000e-01, std::complex<double>(3.829820, 0.000000)},
      {8.750000e-01, std::complex<double>(4.163300, 0.000000)},
      {9.000000e-01, std::complex<double>(4.575060, 0.000000)},
      {9.250000e-01, std::complex<double>(5.112990, 0.000000)},
      {9.500000e-01, std::complex<double>(5.886840, 0.000000)},
      {9.750000e-01, std::complex<double>(7.257360, 0.000000)},
      {9.775000e-01, std::complex<double>(7.471170, 0.000000)},
      {9.800000e-01, std::complex<double>(7.711960, 0.000000)},
      {9.825000e-01, std::complex<double>(7.987180, 0.000000)},
      {9.850000e-01, std::complex<double>(8.307840, 0.000000)},
      {9.875000e-01, std::complex<double>(8.691140, 0.000000)},
      {9.900000e-01, std::complex<double>(9.166180, 0.000000)},
      {9.925000e-01, std::complex<double>(9.788040, 0.000000)},
      {9.950000e-01, std::complex<double>(1.068200e+01, 0.000000)},
      {9.975000e-01, std::complex<double>(1.225460e+01, 0.000000)},
      {9.997500e-01, std::complex<double>(1.780480e+01, 0.000000)},
      {9.999750e-01, std::complex<double>(2.367810e+01, 0.000000)},
      {9.999975e-01, std::complex<double>(2.970570e+01, 0.000000)},
      {1.000002, std::complex<double>(2.965620e+01, 8.318080)},
      {1.000025, std::complex<double>(2.355710e+01, 8.187350)},
      {1.000250, std::complex<double>(1.752540e+01, 7.932950)},
      {1.002500, std::complex<double>(1.173000e+01, 7.356690)},
      {1.005000, std::complex<double>(1.007550e+01, 7.105360)},
      {1.007500, std::complex<double>(9.139160, 6.937030)},
      {1.010000, std::complex<double>(8.490480, 6.813100)},
      {1.012500, std::complex<double>(7.997270, 6.709250)},
      {1.015000, std::complex<double>(7.600540, 6.619270)},
      {1.017500, std::complex<double>(7.270930, 6.544830)},
      {1.020000, std::complex<double>(6.988450, 6.476850)},
      {1.022500, std::complex<double>(6.745280, 6.432950)},
      {1.025000, std::complex<double>(6.508740, 6.302860)},
      {1.050000, std::complex<double>(5.159350, 6.004980)},
      {1.075000, std::complex<double>(4.412310, 5.793880)},
      {1.100000, std::complex<double>(3.905890, 5.647170)},
      {1.125000, std::complex<double>(3.526080, 5.534670)},
      {1.150000, std::complex<double>(3.223860, 5.444410)},
      {1.175000, std::complex<double>(2.975840, 5.370250)},
      {1.200000, std::complex<double>(2.764050, 5.307690)},
      {1.225000, std::complex<double>(2.580530, 5.250960)},
      {1.250000, std::complex<double>(2.420000, 5.203470)},
      {1.275000, std::complex<double>(2.275920, 5.158490)},
      {1.300000, std::complex<double>(2.143720, 5.116090)},
      {1.325000, std::complex<double>(2.029640, 5.081870)},
      {1.350000, std::complex<double>(1.922910, 5.049630)},
      {1.375000, std::complex<double>(1.824490, 5.018220)},
      {1.400000, std::complex<double>(1.733200, 4.989580)},
      {1.425000, std::complex<double>(1.645760, 4.959580)},
      {1.450000, std::complex<double>(1.568710, 4.936700)},
      {1.475000, std::complex<double>(1.496310, 4.913710)},
      {1.500000, std::complex<double>(1.425980, 4.891260)},
      {1.525000, std::complex<double>(1.361940, 4.869200)},
      {1.550000, std::complex<double>(1.281500, 4.851600)},
      {1.575000, std::complex<double>(1.241970, 4.829040)},
      {1.600000, std::complex<double>(1.183670, 4.808110)},
      {1.625000, std::complex<double>(1.129590, 4.788340)},
      {1.650000, std::complex<double>(1.085190, 4.775910)},
      {1.675000, std::complex<double>(1.037810, 4.760570)},
      {1.700000, std::complex<double>(9.923900e-01, 4.743420)},
      {1.725000, std::complex<double>(9.504580e-01, 4.729520)},
      {1.750000, std::complex<double>(9.072510e-01, 4.713500)},
      {1.775000, std::complex<double>(8.679390e-01, 4.698840)},
      {1.800000, std::complex<double>(8.299460e-01, 4.685000)},
      {1.825000, std::complex<double>(7.932470e-01, 4.672120)},
      {1.850000, std::complex<double>(7.576060e-01, 4.658910)},
      {1.875000, std::complex<double>(7.237830e-01, 4.646140)},
      {1.900000, std::complex<double>(6.938680e-01, 4.633010)},
      {1.925000, std::complex<double>(6.595310e-01, 4.622550)},
      {1.950000, std::complex<double>(6.293180e-01, 4.610510)},
      {1.975000, std::complex<double>(5.938350e-01, 4.596510)},
      {2.000000, std::complex<double>(5.708960e-01, 4.588870)},
      {2.025000, std::complex<double>(5.439020e-01, 4.580170)},
      {2.050000, std::complex<double>(5.176490e-01, 4.567900)},
      {2.075000, std::complex<double>(4.926010e-01, 4.557390)},
      {2.100000, std::complex<double>(4.647580e-01, 4.547410)},
      {2.125000, std::complex<double>(4.428880e-01, 4.538930)},
      {2.150000, std::complex<double>(4.199240e-01, 4.528410)},
      {2.175000, std::complex<double>(3.972860e-01, 4.519440)},
      {2.200000, std::complex<double>(3.749200e-01, 4.510430)},
      {2.225000, std::complex<double>(3.507120e-01, 4.500540)},
      {2.250000, std::complex<double>(3.318060e-01, 4.496940)},
      {2.275000, std::complex<double>(3.127840e-01, 4.485150)},
      {2.300000, std::complex<double>(2.923910e-01, 4.476380)},
      {2.325000, std::complex<double>(2.743710e-01, 4.469110)},
      {2.350000, std::complex<double>(2.557950e-01, 4.460490)},
      {2.375000, std::complex<double>(2.374670e-01, 4.451910)},
      {2.400000, std::complex<double>(2.196490e-01, 4.445880)},
      {2.425000, std::complex<double>(2.000050e-01, 4.437940)},
      {2.450000, std::complex<double>(1.849040e-01, 4.430990)},
      {2.475000, std::complex<double>(1.689050e-01, 4.424440)},
      {2.500000, std::complex<double>(1.504040e-01, 4.423210)},
      {2.625000, std::complex<double>(7.934640e-02, 4.381720)},
      {2.750000, std::complex<double>(1.297750e-02, 4.351380)},
      {2.875000, std::complex<double>(-4.733390e-02, 4.323120)},
      {3.000000, std::complex<double>(-1.014980e-01, 4.299410)},
      {3.125000, std::complex<double>(-1.498850e-01, 4.273050)},
      {3.250000, std::complex<double>(-1.938790e-01, 4.250980)},
      {3.375000, std::complex<double>(-2.409990e-01, 4.230170)},
      {3.500000, std::complex<double>(-2.730740e-01, 4.209970)},
      {3.625000, std::complex<double>(-3.080540e-01, 4.192000)},
      {3.750000, std::complex<double>(-3.400120e-01, 4.174700)},
      {3.875000, std::complex<double>(-3.740870e-01, 4.160310)},
      {4.000000, std::complex<double>(-3.973230e-01, 4.144090)},
      {4.125000, std::complex<double>(-4.234040e-01, 4.128620)},
      {4.250000, std::complex<double>(-4.490400e-01, 4.116440)},
      {4.375000, std::complex<double>(-4.700820e-01, 4.101950)},
      {4.500000, std::complex<double>(-4.910300e-01, 4.090400)},
      {4.625000, std::complex<double>(-5.110380e-01, 4.078200)},
      {4.750000, std::complex<double>(-5.297450e-01, 4.067310)},
      {4.875000, std::complex<double>(-5.492750e-01, 4.061330)},
      {5.000000, std::complex<double>(-5.640350e-01, 4.046380)},
      {5.250000, std::complex<double>(-5.947660e-01, 4.027540)},
      {5.500000, std::complex<double>(-6.233500e-01, 4.010660)},
      {5.750000, std::complex<double>(-6.471030e-01, 3.994550)},
      {6.000000, std::complex<double>(-6.695870e-01, 3.979860)},
      {6.250000, std::complex<double>(-6.907760e-01, 3.966670)},
      {6.500000, std::complex<double>(-7.085310e-01, 3.955430)},
      {6.750000, std::complex<double>(-7.254040e-01, 3.943170)},
      {7.000000, std::complex<double>(-7.416220e-01, 3.935370)},
      {7.250000, std::complex<double>(-7.580050e-01, 3.923890)},
      {7.500000, std::complex<double>(-7.689110e-01, 3.913260)},
      {7.750000, std::complex<double>(-7.812950e-01, 3.904020)},
      {8.000000, std::complex<double>(-7.942160e-01, 3.897010)},
      {8.250000, std::complex<double>(-8.078170e-01, 3.832290)},
      {8.500000, std::complex<double>(-8.128800e-01, 3.881760)},
      {8.750000, std::complex<double>(-8.221520e-01, 3.875220)},
      {9.000000, std::complex<double>(-8.304150e-01, 3.868990)},
      {9.250000, std::complex<double>(-8.379850e-01, 3.863090)},
      {9.500000, std::complex<double>(-8.458430e-01, 3.857650)},
      {9.750000, std::complex<double>(-8.528570e-01, 3.852550)},
      {1.000000e+01, std::complex<double>(-8.591450e-01, 3.847710)},
      {1.025000e+01, std::complex<double>(-8.652810e-01, 3.842980)},
      {1.050000e+01, std::complex<double>(-8.689540e-01, 3.844010)},
      {1.075000e+01, std::complex<double>(-8.764360e-01, 3.834700)},
      {1.100000e+01, std::complex<double>(-8.809730e-01, 3.831060)},
      {1.125000e+01, std::complex<double>(-8.860830e-01, 3.827020)},
      {1.150000e+01, std::complex<double>(-8.900930e-01, 3.823440)},
      {1.175000e+01, std::complex<double>(-8.945950e-01, 3.820190)},
      {1.200000e+01, std::complex<double>(-8.984350e-01, 3.817370)},
      {1.225000e+01, std::complex<double>(-9.023230e-01, 3.814250)},
      {1.250000e+01, std::complex<double>(-9.054520e-01, 3.811640)},
      {1.275000e+01, std::complex<double>(-9.102230e-01, 3.809670)},
      {1.300000e+01, std::complex<double>(-9.074140e-01, 3.805780)},
      {1.325000e+01, std::complex<double>(-9.153060e-01, 3.803960)},
      {1.350000e+01, std::complex<double>(-9.177110e-01, 3.801930)},
      {1.375000e+01, std::complex<double>(-9.206080e-01, 3.799770)},
      {1.400000e+01, std::complex<double>(-9.159340e-01, 3.808650)},
      {1.425000e+01, std::complex<double>(-9.244560e-01, 3.799380)},
      {1.450000e+01, std::complex<double>(-9.273640e-01, 3.795870)},
      {1.475000e+01, std::complex<double>(-9.298400e-01, 3.792330)},
      {1.500000e+01, std::complex<double>(-9.310700e-01, 3.790540)},
      {1.525000e+01, std::complex<double>(-9.335660e-01, 3.789160)},
      {1.550000e+01, std::complex<double>(-9.344470e-01, 3.789620)},
      {1.575000e+01, std::complex<double>(-9.365420e-01, 3.787750)},
      {1.600000e+01, std::complex<double>(-9.383150e-01, 3.788630)},
      {1.625000e+01, std::complex<double>(-9.720960e-01, 3.729790)},
      {1.650000e+01, std::complex<double>(-9.410860e-01, 3.783450)},
      {1.675000e+01, std::complex<double>(-9.423010e-01, 3.781260)},
      {1.700000e+01, std::complex<double>(-9.440710e-01, 3.780230)},
      {1.725000e+01, std::complex<double>(-9.431240e-01, 3.790380)},
      {1.750000e+01, std::complex<double>(-9.462580e-01, 3.778470)},
      {1.775000e+01, std::complex<double>(-9.480620e-01, 3.777380)},
      {1.800000e+01, std::complex<double>(-9.486300e-01, 3.776730)},
      {1.825000e+01, std::complex<double>(-9.489330e-01, 3.776790)},
      {1.850000e+01, std::complex<double>(-9.494620e-01, 3.777120)},
      {1.875000e+01, std::complex<double>(-9.509030e-01, 3.774440)},
      {1.900000e+01, std::complex<double>(-9.513830e-01, 3.774230)},
      {1.925000e+01, std::complex<double>(-9.527410e-01, 3.773420)},
      {1.950000e+01, std::complex<double>(-9.529640e-01, 3.772690)},
      {1.975000e+01, std::complex<double>(-9.535020e-01, 3.772610)},
      {2.000000e+01, std::complex<double>(-9.536580e-01, 3.772820)},
      {2.025000e+01, std::complex<double>(-9.551040e-01, 3.771310)},
      {2.050000e+01, std::complex<double>(-9.553750e-01, 3.770870)},
      {2.075000e+01, std::complex<double>(-9.563520e-01, 3.771250)},
      {2.100000e+01, std::complex<double>(-9.558620e-01, 3.770410)},
      {2.125000e+01, std::complex<double>(-9.571490e-01, 3.770990)},
      {2.150000e+01, std::complex<double>(-9.559790e-01, 3.770810)},
      {2.175000e+01, std::complex<double>(-9.573500e-01, 3.769760)},
      {2.200000e+01, std::complex<double>(-9.577910e-01, 3.769160)},
      {2.225000e+01, std::complex<double>(-9.583200e-01, 3.768870)},
      {2.250000e+01, std::complex<double>(-9.550790e-01, 3.776780)},
      {2.275000e+01, std::complex<double>(-9.571400e-01, 3.768270)},
      {2.300000e+01, std::complex<double>(-9.584600e-01, 3.769160)},
      {2.325000e+01, std::complex<double>(-9.585610e-01, 3.767940)},
      {2.350000e+01, std::complex<double>(-9.566560e-01, 3.770740)},
      {2.375000e+01, std::complex<double>(-9.594490e-01, 3.768340)},
      {2.400000e+01, std::complex<double>(-9.580930e-01, 3.767200)},
      {2.425000e+01, std::complex<double>(-9.595470e-01, 3.767990)},
      {2.450000e+01, std::complex<double>(-9.590460e-01, 3.767710)},
      {2.475000e+01, std::complex<double>(-9.596870e-01, 3.768160)},
      {2.500000e+01, std::complex<double>(-9.570650e-01, 3.767160)},
      {2.625000e+01, std::complex<double>(-9.596560e-01, 3.768510)},
      {2.750000e+01, std::complex<double>(-9.590870e-01, 3.769320)},
      {2.875000e+01, std::complex<double>(-9.561250e-01, 3.775530)},
      {3.000000e+01, std::complex<double>(-9.576080e-01, 3.769620)},
      {3.125000e+01, std::complex<double>(-9.531960e-01, 3.779840)},
      {3.250000e+01, std::complex<double>(-9.538940e-01, 3.777550)},
      {3.375000e+01, std::complex<double>(-9.533630e-01, 3.777040)},
      {3.500000e+01, std::complex<double>(-9.517810e-01, 3.778560)},
      {3.625000e+01, std::complex<double>(-9.496690e-01, 3.781310)},
      {3.750000e+01, std::complex<double>(-9.405070e-01, 3.789920)},
      {3.875000e+01, std::complex<double>(-9.458530e-01, 3.785340)},
      {4.000000e+01, std::complex<double>(-9.397520e-01, 3.794720)},
      {4.125000e+01, std::complex<double>(-9.413720e-01, 3.791260)},
      {4.250000e+01, std::complex<double>(-9.323680e-01, 3.808890)},
      {4.375000e+01, std::complex<double>(-9.377840e-01, 3.794770)},
      {4.500000e+01, std::complex<double>(-9.343410e-01, 3.799410)},
      {4.625000e+01, std::complex<double>(-9.327520e-01, 3.801600)},
      {4.750000e+01, std::complex<double>(-9.314380e-01, 3.802120)},
      {4.875000e+01, std::complex<double>(-9.280190e-01, 3.805100)},
      {5.000000e+01, std::complex<double>(-9.265960e-01, 3.807470)},
      {5.250000e+01, std::complex<double>(-9.224410e-01, 3.813370)},
      {5.500000e+01, std::complex<double>(-9.187830e-01, 3.817970)},
      {5.750000e+01, std::complex<double>(-9.135930e-01, 3.824920)},
      {6.000000e+01, std::complex<double>(-9.076430e-01, 3.832210)},
      {6.250000e+01, std::complex<double>(-9.053880e-01, 3.835040)},
      {6.500000e+01, std::complex<double>(-9.000650e-01, 3.842470)},
      {6.750000e+01, std::complex<double>(-8.978360e-01, 3.844550)},
      {7.000000e+01, std::complex<double>(-8.895000e-01, 3.845750)},
      {7.250000e+01, std::complex<double>(-8.896100e-01, 3.851710)},
      {7.500000e+01, std::complex<double>(-8.863290e-01, 3.858890)},
      {7.750000e+01, std::complex<double>(-8.759940e-01, 3.874300)},
      {8.000000e+01, std::complex<double>(-8.772550e-01, 3.869350)},
      {8.250000e+01, std::complex<double>(-8.745030e-01, 3.874770)},
      {8.500000e+01, std::complex<double>(-8.627990e-01, 3.883950)},
      {8.750000e+01, std::complex<double>(-8.685970e-01, 3.883620)},
      {9.000000e+01, std::complex<double>(-8.640790e-01, 3.887640)},
      {9.250000e+01, std::complex<double>(-8.595460e-01, 3.895260)},
      {9.500000e+01, std::complex<double>(-8.576090e-01, 3.896330)},
      {9.750000e+01, std::complex<double>(-8.461040e-01, 3.904570)},
      {1.000000e+02, std::complex<double>(-8.445310e-01, 3.912870)},
      {1.025000e+02, std::complex<double>(-8.185670e-01, 3.922540)},
      {1.050000e+02, std::complex<double>(-8.422780e-01, 3.916480)},
      {1.075000e+02, std::complex<double>(-8.267250e-01, 3.913120)},
      {1.100000e+02, std::complex<double>(-8.360270e-01, 3.924990)},
      {1.125000e+02, std::complex<double>(-8.187290e-01, 3.933470)},
      {1.150000e+02, std::complex<double>(-8.320920e-01, 3.931650)},
      {1.175000e+02, std::complex<double>(-8.624630e-01, 3.891310)},
      {1.200000e+02, std::complex<double>(-8.220970e-01, 3.944540)},
      {1.225000e+02, std::complex<double>(-9.162170e-01, 3.725880)},
      {1.250000e+02, std::complex<double>(-8.206240e-01, 3.949370)},
      {1.375000e+02, std::complex<double>(-8.069580e-01, 3.966440)},
      {1.500000e+02, std::complex<double>(-7.966930e-01, 3.985160)},
      {1.625000e+02, std::complex<double>(-7.618720e-01, 4.012050)},
      {1.750000e+02, std::complex<double>(-7.783480e-01, 4.022170)},
      {1.875000e+02, std::complex<double>(-7.649770e-01, 4.033710)},
      {2.000000e+02, std::complex<double>(-6.854410e-01, 4.114950)},
      {2.125000e+02, std::complex<double>(-7.408410e-01, 4.063980)},
      {2.250000e+02, std::complex<double>(-7.287670e-01, 4.081410)},
      {2.375000e+02, std::complex<double>(-7.135250e-01, 4.100760)},
      {2.500000e+02, std::complex<double>(-7.168470e-01, 4.103730)},
      {2.625000e+02, std::complex<double>(-7.527330e-01, 4.072580)},
      {2.750000e+02, std::complex<double>(-7.124000e-01, 4.120630)},
      {2.875000e+02, std::complex<double>(-7.035340e-01, 4.135590)},
      {3.000000e+02, std::complex<double>(-6.971670e-01, 4.141230)},
      {3.125000e+02, std::complex<double>(-6.910640e-01, 4.156440)},
      {3.250000e+02, std::complex<double>(-6.778280e-01, 4.178490)},
      {3.375000e+02, std::complex<double>(-6.677270e-01, 4.175210)},
      {3.500000e+02, std::complex<double>(-6.594620e-01, 4.170790)},
      {3.625000e+02, std::complex<double>(-6.689900e-01, 4.193900)},
      {3.750000e+02, std::complex<double>(-6.502250e-01, 4.205840)},
      {3.875000e+02, std::complex<double>(-6.711730e-01, 4.216770)},
      {4.000000e+02, std::complex<double>(-5.948230e-01, 4.204950)},
      {4.125000e+02, std::complex<double>(-6.417410e-01, 4.226540)},
      {4.250000e+02, std::complex<double>(-6.472120e-01, 4.235860)},
      {4.375000e+02, std::complex<double>(-6.412130e-01, 4.247910)},
      {4.500000e+02, std::complex<double>(-6.151300e-01, 4.239300)},
      {4.625000e+02, std::complex<double>(-6.000860e-01, 4.235240)},
      {4.750000e+02, std::complex<double>(-4.474580e-01, 4.420810)},
      {4.875000e+02, std::complex<double>(-6.416880e-01, 4.279580)},
      {5.000000e+02, std::complex<double>(-6.278020e-01, 4.282370)},
      {5.250000e+02, std::complex<double>(-6.065990e-01, 4.302910)},
      {5.500000e+02, std::complex<double>(-6.220060e-01, 4.310840)},
      {5.750000e+02, std::complex<double>(-5.434380e-01, 4.340190)},
      {6.000000e+02, std::complex<double>(-6.171970e-01, 4.329800)},
      {6.250000e+02, std::complex<double>(-5.839680e-01, 4.359710)},
      {6.500000e+02, std::complex<double>(-5.466400e-01, 4.303160)},
      {6.750000e+02, std::complex<double>(-5.846380e-01, 4.370330)},
      {7.000000e+02, std::complex<double>(-5.940230e-01, 4.364990)},
      {7.250000e+02, std::complex<double>(-5.928400e-01, 4.385600)},
      {7.500000e+02, std::complex<double>(-5.856290e-01, 4.384290)},
      {7.750000e+02, std::complex<double>(-5.833640e-01, 4.389110)},
      {8.000000e+02, std::complex<double>(-5.596990e-01, 4.417590)},
      {8.250000e+02, std::complex<double>(-5.590650e-01, 4.434450)},
      {8.500000e+02, std::complex<double>(-6.216490e-01, 4.371060)},
      {8.750000e+02, std::complex<double>(-5.661810e-01, 4.444410)},
      {9.000000e+02, std::complex<double>(-5.675510e-01, 4.443180)},
      {9.250000e+02, std::complex<double>(-5.157180e-01, 4.376070)},
      {9.500000e+02, std::complex<double>(-5.660980e-01, 4.473550)},
      {9.750000e+02, std::complex<double>(-5.434320e-01, 4.493080)},
      {1.000000e+03, std::complex<double>(-5.233450e-01, 4.482900)},
      {1.025000e+03, std::complex<double>(-5.585990e-01, 4.496640)},
      {1.050000e+03, std::complex<double>(-5.561640e-01, 4.504080)},
      {1.075000e+03, std::complex<double>(-5.539260e-01, 4.511270)},
      {1.100000e+03, std::complex<double>(-5.672790e-01, 4.482900)},
      {1.125000e+03, std::complex<double>(-5.788380e-01, 4.482020)},
      {1.150000e+03, std::complex<double>(-5.477980e-01, 4.531840)},
      {1.175000e+03, std::complex<double>(-4.818960e-01, 4.579170)},
      {1.200000e+03, std::complex<double>(-4.779800e-01, 4.461080)},
      {1.225000e+03, std::complex<double>(-4.830520e-01, 4.552330)},
      {1.250000e+03, std::complex<double>(-5.402060e-01, 4.557730)},
      {1.375000e+03, std::complex<double>(-5.028850e-01, 4.594370)},
      {1.500000e+03, std::complex<double>(-5.251460e-01, 4.614700)},
      {1.625000e+03, std::complex<double>(-5.561430e-01, 4.654850)},
      {1.750000e+03, std::complex<double>(-5.137850e-01, 4.663500)},
      {1.875000e+03, std::complex<double>(-4.949940e-01, 4.711330)},
      {2.000000e+03, std::complex<double>(-4.878720e-01, 4.742990)},
      {2.125000e+03, std::complex<double>(-6.820930e-01, 4.567030)},
      {2.250000e+03, std::complex<double>(-4.981180e-01, 4.744200)},
      {2.375000e+03, std::complex<double>(-4.952330e-01, 4.761730)},
      {2.500000e+03, std::complex<double>(-4.926530e-01, 4.778410)},
      {2.625000e+03, std::complex<double>(-4.903560e-01, 4.794340)},
      {2.750000e+03, std::complex<double>(-4.882910e-01, 4.809550)},
      {2.875000e+03, std::complex<double>(-4.864500e-01, 4.824130)},
      {3.000000e+03, std::complex<double>(-4.848020e-01, 4.838120)},
      {3.125000e+03, std::complex<double>(-4.833320e-01, 4.851570)},
      {3.250000e+03, std::complex<double>(-4.820140e-01, 4.864520)},
      {3.375000e+03, std::complex<double>(-4.808430e-01, 4.877010)},
      {3.500000e+03, std::complex<double>(-4.797980e-01, 4.889060)},
      {3.625000e+03, std::complex<double>(-4.788770e-01, 4.900710)},
      {3.750000e+03, std::complex<double>(-4.780630e-01, 4.911980)},
      {3.875000e+03, std::complex<double>(-4.773460e-01, 4.922910)},
      {4.000000e+03, std::complex<double>(-4.767180e-01, 4.933500)},
      {4.125000e+03, std::complex<double>(-4.761750e-01, 4.943780)},
      {4.250000e+03, std::complex<double>(-4.757100e-01, 4.953760)},
      {4.375000e+03, std::complex<double>(-4.753040e-01, 4.963460)},
      {4.500000e+03, std::complex<double>(-4.749280e-01, 4.972830)},
      {4.625000e+03, std::complex<double>(-4.747220e-01, 4.982120)},
      {4.750000e+03, std::complex<double>(-4.745060e-01, 4.991080)},
      {4.875000e+03, std::complex<double>(-4.743460e-01, 4.999830)},
      {5.000000e+03, std::complex<double>(-4.742420e-01, 5.008360)},
      {5.250000e+03, std::complex<double>(-4.741500e-01, 5.024820)},
      {5.500000e+03, std::complex<double>(-4.742210e-01, 5.040540)},
      {5.750000e+03, std::complex<double>(-4.744350e-01, 5.055600)},
      {6.000000e+03, std::complex<double>(-4.747660e-01, 5.070030)},
      {6.250000e+03, std::complex<double>(-4.752050e-01, 5.083900)},
      {6.500000e+03, std::complex<double>(-4.757360e-01, 5.097240)},
      {6.750000e+03, std::complex<double>(-4.763560e-01, 5.110090)},
      {7.000000e+03, std::complex<double>(-4.770560e-01, 5.122500)},
      {7.250000e+03, std::complex<double>(-4.778150e-01, 5.134490)},
      {7.500000e+03, std::complex<double>(-4.786300e-01, 5.146070)},
      {7.750000e+03, std::complex<double>(-4.795060e-01, 5.157300)},
      {8.000000e+03, std::complex<double>(-4.804330e-01, 5.168170)},
      {8.250000e+03, std::complex<double>(-4.813910e-01, 5.178730)},
      {8.500000e+03, std::complex<double>(-4.823970e-01, 5.188980)},
      {8.750000e+03, std::complex<double>(-4.834230e-01, 5.198940)},
      {9.000000e+03, std::complex<double>(-4.844930e-01, 5.208620)},
      {9.250000e+03, std::complex<double>(-4.855350e-01, 5.218100)},
      {9.500000e+03, std::complex<double>(-4.866970e-01, 5.227250)},
      {9.750000e+03, std::complex<double>(-4.878450e-01, 5.236200)},
      {1.000000e+04, std::complex<double>(-4.890060e-01, 5.244930)},
      {1.025000e+04, std::complex<double>(-4.901930e-01, 5.253470)},
      {1.050000e+04, std::complex<double>(-4.913890e-01, 5.261780)},
      {1.075000e+04, std::complex<double>(-4.925970e-01, 5.269930)},
      {1.100000e+04, std::complex<double>(-4.938190e-01, 5.277880)},
      {1.125000e+04, std::complex<double>(-4.950610e-01, 5.285640)},
      {1.150000e+04, std::complex<double>(-4.963020e-01, 5.293260)},
      {1.175000e+04, std::complex<double>(-4.975540e-01, 5.300720)},
      {1.200000e+04, std::complex<double>(-4.988200e-01, 5.308020)},
      {1.225000e+04, std::complex<double>(-5.000850e-01, 5.315170)},
      {1.250000e+04, std::complex<double>(-5.013610e-01, 5.322180)},
      {1.375000e+04, std::complex<double>(-5.078200e-01, 5.355320)},
      {1.500000e+04, std::complex<double>(-5.143100e-01, 5.385630)},
      {1.625000e+04, std::complex<double>(-5.208080e-01, 5.413580)},
      {1.750000e+04, std::complex<double>(-5.258070e-01, 5.440400)},
      {1.875000e+04, std::complex<double>(-5.337290e-01, 5.463660)},
      {2.000000e+04, std::complex<double>(-5.400840e-01, 5.486300)},
      {2.125000e+04, std::complex<double>(-5.446100e-01, 5.508630)},
      {2.250000e+04, std::complex<double>(-5.485740e-01, 5.525210)},
      {2.375000e+04, std::complex<double>(-5.562090e-01, 5.543070)},
      {2.500000e+04, std::complex<double>(-5.248800e-01, 5.588220)},
      {2.625000e+04, std::complex<double>(-5.674120e-01, 5.585320)},
      {2.750000e+04, std::complex<double>(-5.731940e-01, 5.601780)},
      {2.875000e+04, std::complex<double>(-5.789180e-01, 5.617530)},
      {3.000000e+04, std::complex<double>(-5.845250e-01, 5.632620)},
      {3.125000e+04, std::complex<double>(-5.900550e-01, 5.647070)},
      {3.250000e+04, std::complex<double>(-5.954660e-01, 5.661010)},
      {3.375000e+04, std::complex<double>(-6.008260e-01, 5.674400)},
      {3.500000e+04, std::complex<double>(-6.060870e-01, 5.687340)},
      {3.625000e+04, std::complex<double>(-6.112750e-01, 5.699800)},
      {3.750000e+04, std::complex<double>(-6.164020e-01, 5.711880)},
      {3.875000e+04, std::complex<double>(-6.214450e-01, 5.723540)},
      {4.000000e+04, std::complex<double>(-6.263970e-01, 5.734860)},
      {4.125000e+04, std::complex<double>(-6.312700e-01, 5.745830)},
      {4.250000e+04, std::complex<double>(-6.360560e-01, 5.756480)},
      {4.375000e+04, std::complex<double>(-6.408450e-01, 5.766830)},
      {4.500000e+04, std::complex<double>(-6.457050e-01, 5.776760)},
      {4.625000e+04, std::complex<double>(-6.503670e-01, 5.786530)},
      {4.750000e+04, std::complex<double>(-6.547850e-01, 5.796100)},
      {4.875000e+04, std::complex<double>(-6.594220e-01, 5.805320)},
      {5.000000e+04, std::complex<double>(-6.637540e-01, 5.814450)},
      {5.250000e+04, std::complex<double>(-6.724940e-01, 5.831850)},
      {5.500000e+04, std::complex<double>(-6.806730e-01, 5.848680)},
      {5.750000e+04, std::complex<double>(-6.889350e-01, 5.864560)},
      {6.000000e+04, std::complex<double>(-6.973210e-01, 5.879680)},
      {6.250000e+04, std::complex<double>(-7.053950e-01, 5.894190)},
      {6.500000e+04, std::complex<double>(-7.130550e-01, 5.908250)},
      {6.750000e+04, std::complex<double>(-7.202690e-01, 5.921980)},
      {7.000000e+04, std::complex<double>(-7.279810e-01, 5.934910)},
      {7.250000e+04, std::complex<double>(-7.349080e-01, 5.947620)},
      {7.500000e+04, std::complex<double>(-7.421590e-01, 5.959740)},
      {7.750000e+04, std::complex<double>(-7.489310e-01, 5.971620)},
      {8.000000e+04, std::complex<double>(-7.558540e-01, 5.982990)},
      {8.250000e+04, std::complex<double>(-7.630040e-01, 5.993840)},
      {8.500000e+04, std::complex<double>(-7.692540e-01, 6.004730)},
      {8.750000e+04, std::complex<double>(-7.760400e-01, 6.015000)},
      {9.000000e+04, std::complex<double>(-7.819260e-01, 6.025360)},
      {9.250000e+04, std::complex<double>(-7.883920e-01, 6.035080)},
      {9.500000e+04, std::complex<double>(-7.945700e-01, 6.044670)},
      {9.750000e+04, std::complex<double>(-8.002950e-01, 6.054170)},
      {1.000000e+05, std::complex<double>(-8.060720e-01, 6.063380)},
      {1.025000e+05, std::complex<double>(-8.123860e-01, 6.072040)},
      {1.050000e+05, std::complex<double>(-8.184380e-01, 6.080510)},
      {1.075000e+05, std::complex<double>(-8.236750e-01, 6.089250)},
      {1.100000e+05, std::complex<double>(-8.293480e-01, 6.097460)},
      {1.125000e+05, std::complex<double>(-8.347930e-01, 6.105660)},
      {1.150000e+05, std::complex<double>(-8.400500e-01, 6.113680)},
      {1.175000e+05, std::complex<double>(-8.458380e-01, 6.121140)},
      {1.200000e+05, std::complex<double>(-8.505530e-01, 6.129080)},
      {1.225000e+05, std::complex<double>(-8.564450e-01, 6.136180)},
      {1.250000e+05, std::complex<double>(-8.614150e-01, 6.143510)},
      {1.375000e+05, std::complex<double>(-8.858380e-01, 6.178060)},
      {1.500000e+05, std::complex<double>(-9.091280e-01, 6.209430)},
      {1.625000e+05, std::complex<double>(-9.310480e-01, 6.238410)},
      {1.750000e+05, std::complex<double>(-9.516100e-01, 6.265350)},
      {1.875000e+05, std::complex<double>(-9.717480e-01, 6.290160)},
      {2.000000e+05, std::complex<double>(-9.903380e-01, 6.313530)},
      {2.125000e+05, std::complex<double>(-1.008530, 6.335540)},
      {2.250000e+05, std::complex<double>(-1.025850, 6.356120)},
      {2.375000e+05, std::complex<double>(-1.042680, 6.375730)},
      {2.500000e+05, std::complex<double>(-1.040430, 6.358410)}
   };

   return data;
}

} // namespace effective_couplings

} // namespace flexiblesusy
